/** 
 *   vol_get_point: Extraer los datos de una coordenada, con sus
 *   puntos linderos o todo el volumen.
 *
 *   (c) Copyright 2009 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <getopt.h>
#include <fcntl.h>
#include <my_stdio.h>


#include "version.h"
#include "my_xcomp.h"
#include "blob.h"
#include "xml_proc.h"
#include "kdp2dbz_func.h"
#include "ang_tools.h"
#include "get_point.h"
#include "geo.h"


const char *name_definition = {"Vol extractor"};
static const char *help_text[] ={
	"OPTIONS:",
#       ifdef VERBOSE_MODE
        "  -v\t\t - Verbose mode (incremental).",
#       endif
        "  -V\t\t - Show program version.",
	"  -o\t\t - File out (default stdout).",
	"  --lat\t\t - Latitude of point (mandatory or --all).",
	"  --lon\t\t - Longitude of point (mandatory or --all).",
	"  --nodata\t - Use it for no data values.",
	"  --griddim\t - Dimension of data grid out, def: 3 (use odd value)",
	"  --time\t - Put time tag in output file.",
	"  --posang\t - Put pos angle (elevation) tag in output file.",
	"  --pixid\t - Put pix id num tag in output file.",
	"  --tags\t - Put time, pos angle and pix id in output file.",
	"  --add\t\t - Add out to an existing output file.",
	"  --all\t\t - Extract all points (inhibit --lat and --lon).",
        "  -h [--help]\t - This help !!!.",
        ""
};

void version(char *program_name)
{
        fprintf(stderr,"%s: ", name_definition);
        fprintf(stderr,"%s version %s\n", program_name, VERSION);
}


void usage(char *program_name)
{
        version(program_name);
        fprintf(stderr,
 "Usage: \t%s [OPTIONS] (--lat lat --lon lon | --all) (src_file_name|-)\n",
                program_name);
}


void help()
{
        int i=0;

        while((char) *help_text[i] != 0)
                fprintf(stderr,"%s\n",help_text[i++]);
        fprintf(stderr,"\n");
}


int main(int argc, char **argv)
{
	struct blob *bl = NULL;
	XML_Parser parser = xmlp_init();
        char *program_name;
	int blob_count;
	char *file_in = NULL;
	char *file_out = NULL;
        char *cp;
	int op;
	double p_lat;
	double p_lon;
	double distance;
	double bearing;
	struct gp_param param = {
		.time_flag = 0,
		.posangle_flag = 0,
		.pix_num_flag = 0,
		.pixel_grid = 3, /* default 3x3*/
		.no_data = NULL,
		.no_data_out_flag = 1,
	};
	int add_to_out_flag = 0;
	int all_points_flag = 0;
	int ret = 0;
        struct option const longopts[] =
        {
                {"help", 0, 0, 'h'},
                {"verbose", 0, 0, 'v'},
		{"debug", 0, 0, 201},
		{"lat", 1, 0, 202},
		{"lon", 1, 0, 203},
		{"time", 0, 0, 204},
		{"nodata", 1, 0, 205},
		{"griddim", 1, 0, 206},
		{"posang", 0, 0, 207},
		{"pixid", 0, 0, 208},
		{"tags", 0, 0, 209},
		{"add", 0, 0, 210},
		{"all", 0, 0, 211},
                {0,  0,  0,  0}
        };

        /* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];

	p_lat = p_lon = 361.0;
	debug_flag = 0;
        verbose_flag = PERR;
        while ((op = getopt_long(argc, argv, "vhVo:",longopts, 0)) != EOF)
                switch( op ){
		case 'o':
			file_out = optarg;
			if (strlen(optarg) == 1 && *(optarg) == '-')
				file_out = NULL;
			break;
		case 201:
			debug_flag = 1;
			break;
		case 202:
			p_lat = strdeg2double(optarg);;
			break;
		case 203:
			p_lon = strdeg2double(optarg);;
			break;
		case 204:
			param.time_flag = 1;
			break;
		case 205:
			asprintf(&param.no_data, "%s", optarg);
			break;
		case 206:
			param.pixel_grid = 0x1 | atoi(optarg);
			break;
		case 207:
			param.posangle_flag = 1;
			break;
		case 208:
			param.pix_num_flag = 1;
			break;
		case 209:
			param.pix_num_flag = 1;
			param.posangle_flag = 1;
			param.time_flag = 1;
			break;
		case 210:
			add_to_out_flag = 1;
			break;
		case 211:
			add_to_out_flag = 0;
			all_points_flag = 1;
			/* TODO */
			
			break;
                case 'v':
                        verbose_flag = (verbose_flag << 1) | 1;
                        break;
                case 'V':
                        version(program_name);
                        exit(1);
                        break;
                case 'h':
			usage(program_name);
                        help();
                        exit(1);
                default:
                        usage(program_name);
                        exit(1);
                }

	if ( (p_lat == 361.0 || p_lon == 361.0) && (all_points_flag == 0)) {
		fprintf(stderr, "Poit not defined.\n");
		return 1;
	}
	file_in = argv[optind];
	if ((blob_count = blob_get_file(file_in, &bl)) < 1 ) {
		fprintf(stderr,"Bad input file: %s\n", file_in);
		return 1;
	}
	if ( param.no_data == NULL )
		asprintf(&param.no_data, "-");
	else {
		if (strncmp(param.no_data, "exclude", 7) == 0)
			param.no_data_out_flag = 0;
	}
	XMLP_SET_BLOB(parser, bl);
	XMLP_SET_INIVIT_OUT_FLAG(parser, 1);
	if ( add_to_out_flag )
		XMLP_GET_ADD_OUT_FILE(parser) = 1;
	if (file_out == NULL)
		XMLP_SET_FPO(parser, stdout);
	else
		xmlp_open_file_out(parser, file_out);
	xmlp_run_on(parser, file_in);

	if (all_points_flag == 0) {
		distance = geo_distance( XMLP_GET_LAT(parser) * _DEG2RAD,
					 XMLP_GET_LON(parser) * _DEG2RAD,
					 p_lat * _DEG2RAD,
					 p_lon * _DEG2RAD);
		bearing = mod360( geo_bearing(XMLP_GET_LAT(parser) * _DEG2RAD,
					      XMLP_GET_LON(parser) * _DEG2RAD,
					      p_lat * _DEG2RAD,
					      p_lon * _DEG2RAD) / _DEG2RAD);
#if 1
	PDBG("Lat: %f -> %f", XMLP_GET_LAT(parser), p_lat);
	PDBG("Lon: %f -> %f", XMLP_GET_LON(parser), p_lon);
	PDBG("Alt: %f", XMLP_GET_ALT(parser));
	PDBG("ID : %s", XMLP_GET_RIID(parser));
	PDBG("Stop Range: %d", XMLP_GET_STOPRANGE(parser));
	PDBG("Beam Width: %f", XMLP_GET_BEAMWIDTH(parser));
	PDBG("Time: %ld - %s", XMLP_GET_TIME(parser),
	     asctime(&XMLP_GET_TM(parser)));
	PDBG("Distance: %f Km", distance);
	PDBG("Bearing: %f degrees", bearing);
#endif
	if (distance < XMLP_GET_STOPRANGE(parser)) {
		param.tm = &XMLP_GET_TM(parser);
		param.stop_range = XMLP_GET_STOPRANGE(parser);
		blob_find_pos(XMLP_GET_FPO(parser), bl, &param,
			      distance, bearing +0.0);
	} else {
		fprintf(stderr, "Out of reange: %.1f Km\n", distance); 
		ret = 2;
	}
	} else {
		param.stop_range = XMLP_GET_STOPRANGE(parser);
		param.r_lat = XMLP_GET_LAT(parser) * _DEG2RAD;
		param.r_lon = XMLP_GET_LON(parser) * _DEG2RAD;
		blob_dump_pos(XMLP_GET_FPO(parser), bl, &param,
			      XMLP_GET_LAT(parser), XMLP_GET_LON(parser));
	}
	if (file_out != NULL)
		xmlp_close_file_out(parser);

	xmlp_free(parser);
	free_blob_all(bl);
	free(param.no_data);
	
	return ret;
}
	
		
	
